import enum

class EnvironmentVariablesEnum(enum.Enum):
    HAPE_LOG_FILE = "HAPE_LOG_FILE"
    HAPE_LOG_LEVEL = "HAPE_LOG_LEVEL"
    HAPE_LOG_ROTATE_EVERY_RUN = "HAPE_LOG_ROTATE_EVERY_RUN"
    HAPE_MARIADB_HOST = "HAPE_MARIADB_HOST"
    HAPE_MARIADB_USERNAME = "HAPE_MARIADB_USERNAME"
    HAPE_MARIADB_PASSWORD = "HAPE_MARIADB_PASSWORD"
    HAPE_MARIADB_DATABASE = "HAPE_MARIADB_DATABASE"
    HAPE_GITLAB_DOMAIN = "HAPE_GITLAB_DOMAIN"
    HAPE_GITLAB_TOKEN = "HAPE_GITLAB_TOKEN"
