import enum

class CrudColumnValidTypesEnum(enum.Enum):
    STRING = "string"
    TEXT = "text"
    INT = "int"
    BOOL = "bool"
    FLOAT = "float"
    DATE = "date"
    DATETIME = "datetime"
    TIMESTAMP = "timestamp"
